@@ -4,7 +4,7 @@
 /*
  * strftime.c
  * Original Author:	G. Haley
- * Additions from:	Eric Blake
+ * Additions from:	Eric Blake, Corinna Vinschen
  * Changes to allow dual use as wcstime, also:	Craig Howland
  *
  * Places characters into the array pointed to by s as controlled by the string
@@ -17,30 +17,33 @@
 
 /*
 FUNCTION
-<<strftime>>---convert date and time to a formatted string
+<<strftime>>, <<strftime_l>>---convert date and time to a formatted string
 
 INDEX
 	strftime
 
-ANSI_SYNOPSIS
+INDEX
+	strftime_l
+
+SYNOPSIS
 	#include <time.h>
 	size_t strftime(char *restrict <[s]>, size_t <[maxsize]>,
 			const char *restrict <[format]>,
                         const struct tm *restrict <[timp]>);
-
-TRAD_SYNOPSIS
-	#include <time.h>
-	size_t strftime(<[s]>, <[maxsize]>, <[format]>, <[timp]>)
-	char *<[s]>;
-	size_t <[maxsize]>;
-	char *<[format]>;
-	struct tm *<[timp]>;
+	size_t strftime_l(char *restrict <[s]>, size_t <[maxsize]>,
+			  const char *restrict <[format]>,
+			  const struct tm *restrict <[timp]>,
+			  locale_t <[locale]>);
 
 DESCRIPTION
 <<strftime>> converts a <<struct tm>> representation of the time (at
 <[timp]>) into a null-terminated string, starting at <[s]> and occupying
 no more than <[maxsize]> characters.
 
+<<strftime_l>> is like <<strftime>> but creates a string in a format
+as expected in locale <[locale]>.  If <[locale]> is LC_GLOBAL_LOCALE or
+not a valid locale object, the behaviour is undefined.
+
 You control the format of the output using the string at <[format]>.
 <<*<[format]>>> can contain two kinds of specifications: text to be
 copied literally into the formatted string, and time conversion
@@ -79,7 +82,7 @@ The century, that is, the year divided by 100 then truncated.  For
 4-digit years, the result is zero-padded and exactly two characters;
 but for other years, there may a negative sign or more digits.  In
 this way, `<<%C%y>>' is equivalent to `<<%Y>>'. [tm_year]
- 
+
 o %d
 The day of the month, formatted with two digits (from `<<01>>' to
 `<<31>>'). [tm_mday]
@@ -110,7 +113,7 @@ includes January 4th, and begin on Mondays. Therefore, if January 1st,
 2nd, or 3rd falls on a Sunday, that day and earlier belong to the last
 week of the previous year; and if December 29th, 30th, or 31st falls
 on Monday, that day and later belong to week 1 of the next year.  For
-consistency with %Y, it always has at least four characters. 
+consistency with %Y, it always has at least four characters.
 Example: "%G" for Saturday 2nd January 1999 gives "1998", and for
 Tuesday 30th December 1997 gives "1998". [tm_year, tm_wday, tm_yday]
 
@@ -166,6 +169,10 @@ notations, the result is an empty string. [tm_sec, tm_min, tm_hour]
 o %R
 The 24-hour time, to the minute.  Equivalent to "%H:%M". [tm_min, tm_hour]
 
+o %s
+The time elapsed, in seconds, since the start of the Unix epoch at
+1970-01-01 00:00:00 UTC.
+
 o %S
 The second, formatted with two digits (from `<<00>>' to `<<60>>').  The
 value 60 accounts for the occasional leap second. [tm_sec]
@@ -254,11 +261,13 @@ value beforehand to distinguish between failure and an empty string.
 This implementation does not support <<s>> being NULL, nor overlapping
 <<s>> and <<format>>.
 
-<<strftime>> requires no supporting OS subroutines.
+<<strftime_l>> is POSIX-1.2008.
+
+<<strftime>> and <<strftime_l>> require no supporting OS subroutines.
 
 BUGS
-<<strftime>> ignores the LC_TIME category of the current locale, hard-coding
-the "C" locale settings.
+(NOT Cygwin:) <<strftime>> ignores the LC_TIME category of the current
+locale, hard-coding the "C" locale settings.
 */
 
 #include <newlib.h>
@@ -272,8 +281,8 @@ the "C" locale settings.
 #include <ctype.h>
 #include <wctype.h>
 #include "local.h"
-#include "../locale/timelocal.h"
- 
+#include "../locale/setlocale.h"
+
 /* Defines to make the file dual use for either strftime() or wcsftime().
  * To get wcsftime, define MAKE_WCSFTIME.
  * To get strftime, do not define MAKE_WCSFTIME.
@@ -294,6 +303,7 @@ the "C" locale settings.
 #  define STRLEN(a)	strlen(a)
 # else
 #  define strftime	wcsftime	/* Alternate function name */
+#  define strftime_l	wcsftime_l	/* Alternate function name */
 #  define CHAR		wchar_t		/* string type basis */
 #  define CQ(a)		L##a		/* character constant qualifier */
 #  define snprintf	swprintf	/* wide-char equivalent function name */
@@ -331,15 +341,14 @@ the "C" locale settings.
 #  error "YEAR_BASE < 0"
 #endif
 
-static _CONST int dname_len[7] =
+static const int dname_len[7] =
 {6, 6, 7, 9, 8, 6, 8};
 
 /* Using the tm_year, tm_wday, and tm_yday components of TIM_P, return
    -1, 0, or 1 as the adjustment to add to the year for the ISO week
    numbering used in "%g%G%V", avoiding overflow.  */
 static int
-_DEFUN (iso_year_adjust, (tim_p),
-	_CONST struct tm *tim_p)
+iso_year_adjust (const struct tm *tim_p)
 {
   /* Account for fact that tm_year==0 is year 1900.  */
   int leap = isleap (tim_p->tm_year + (YEAR_BASE
@@ -590,7 +599,7 @@ get_alt_digits (const char *alt_digits)
       ++adi->num;
   /* Allocate the `digit' array, which is an array of `num' pointers into
      `buffer'. */
-  adi->digit = (CHAR **) calloc (adi->num, sizeof (CHAR **));
+  adi->digit = (CHAR **) calloc (adi->num, sizeof (CHAR *));
   if (!adi->digit)
     {
       free (adi);
@@ -658,53 +667,31 @@ conv_to_alt_digits (CHAR *buf, size_t bufsiz, unsigned num, alt_digits_t *adi)
   return 0;
 }
 
-static size_t __strftime (CHAR *, size_t, const CHAR *, const struct tm *,
-			  era_info_t **, alt_digits_t **);
-
-size_t
-_DEFUN (strftime, (s, maxsize, format, tim_p),
-	CHAR *__restrict s _AND
-	size_t maxsize _AND
-	_CONST CHAR *__restrict format _AND
-	_CONST struct tm *__restrict tim_p)
-{
-  era_info_t *era_info = NULL;
-  alt_digits_t *alt_digits = NULL;
-  size_t ret = __strftime (s, maxsize, format, tim_p, &era_info, &alt_digits);
-  if (era_info)
-    free_era_info (era_info);
-  if (alt_digits)
-    free_alt_digits (alt_digits);
-  return ret;
-}
-
 static size_t
 __strftime (CHAR *s, size_t maxsize, const CHAR *format,
-	    const struct tm *tim_p, era_info_t **era_info,
-	    alt_digits_t **alt_digits)
+	    const struct tm *tim_p, struct __locale_t *locale,
+	    era_info_t **era_info, alt_digits_t **alt_digits)
 #else /* !_WANT_C99_TIME_FORMATS */
-# define __strftime(s,m,f,t,e,a)	strftime((s),(m),(f),(t))
+static size_t
+__strftime (CHAR *s, size_t maxsize, const CHAR *format,
+	    const struct tm *tim_p, struct __locale_t *locale)
 
-size_t
-_DEFUN (strftime, (s, maxsize, format, tim_p),
-	CHAR *__restrict s _AND
-	size_t maxsize _AND
-	_CONST CHAR *__restrict format _AND
-	_CONST struct tm *__restrict tim_p)
+#define __strftime(s,m,f,t,l,e,a)	__strftime((s),(m),(f),(t),(l))
 #endif /* !_WANT_C99_TIME_FORMATS */
 {
   size_t count = 0;
-  int i, len = 0;
+  int len = 0;
   const CHAR *ctloc;
 #if defined (MAKE_WCSFTIME) && !defined (__HAVE_LOCALE_INFO_EXTENDED__)
   CHAR ctlocbuf[CTLOCBUFLEN];
 #endif
-  size_t ctloclen;
+  size_t i, ctloclen;
   CHAR alt;
   CHAR pad;
   unsigned long width;
+  int tzset_called = 0;
 
-  struct lc_time_T *_CurrentTimeLocale = __get_current_time_locale ();
+  const struct lc_time_T *_CurrentTimeLocale = __get_time_locale (locale);
   for (;;)
     {
       while (*format && *format != CQ('%'))
@@ -737,7 +724,7 @@ _DEFUN (strftime, (s, maxsize, format, tim_p),
       if (*format == CQ('E'))
 	{
 	  alt = *format++;
-#ifdef _WANT_C99_TIME_FORMATS      
+#ifdef _WANT_C99_TIME_FORMATS
 #if defined (MAKE_WCSFTIME) && defined (__HAVE_LOCALE_INFO_EXTENDED__)
 	  if (!*era_info && *_CurrentTimeLocale->wera)
 	    *era_info = get_era_info (tim_p, _CurrentTimeLocale->wera);
@@ -750,7 +737,7 @@ _DEFUN (strftime, (s, maxsize, format, tim_p),
       else if (*format == CQ('O'))
 	{
 	  alt = *format++;
-#ifdef _WANT_C99_TIME_FORMATS      
+#ifdef _WANT_C99_TIME_FORMATS
 #if defined (MAKE_WCSFTIME) && defined (__HAVE_LOCALE_INFO_EXTENDED__)
 	  if (!*alt_digits && *_CurrentTimeLocale->walt_digits)
 	    *alt_digits = get_alt_digits (_CurrentTimeLocale->walt_digits);
@@ -835,7 +822,7 @@ recurse:
 	    {
 	      /* Recurse to avoid need to replicate %Y formation. */
 	      len = __strftime (&s[count], maxsize - count, ctloc, tim_p,
-				era_info, alt_digits);
+				locale, era_info, alt_digits);
 	      if (len > 0)
 		count += len;
 	      else
@@ -933,7 +920,7 @@ recurse:
 	  { /* %F is equivalent to "%+4Y-%m-%d", flags and width can change
 	       that.  Recurse to avoid need to replicate %Y formation. */
 	    CHAR fmtbuf[32], *fmt = fmtbuf;
-	    
+
 	    *fmt++ = CQ('%');
 	    if (pad) /* '0' or '+' */
 	      *fmt++ = pad;
@@ -952,7 +939,7 @@ recurse:
 	      }
 	    STRCPY (fmt, CQ("Y-%m-%d"));
 	    len = __strftime (&s[count], maxsize - count, fmtbuf, tim_p,
-			      era_info, alt_digits);
+			      locale, era_info, alt_digits);
 	    if (len > 0)
 	      count += len;
 	    else
@@ -1108,6 +1095,74 @@ recurse:
 			  tim_p->tm_hour, tim_p->tm_min);
           CHECK_LENGTH ();
           break;
+	case CQ('s'):
+/*
+ * From:
+ * The Open Group Base Specifications Issue 7
+ * IEEE Std 1003.1, 2013 Edition
+ * Copyright (c) 2001-2013 The IEEE and The Open Group
+ * XBD Base Definitions
+ * 4. General Concepts
+ * 4.15 Seconds Since the Epoch
+ * A value that approximates the number of seconds that have elapsed since the
+ * Epoch. A Coordinated Universal Time name (specified in terms of seconds
+ * (tm_sec), minutes (tm_min), hours (tm_hour), days since January 1 of the year
+ * (tm_yday), and calendar year minus 1900 (tm_year)) is related to a time
+ * represented as seconds since the Epoch, according to the expression below.
+ * If the year is <1970 or the value is negative, the relationship is undefined.
+ * If the year is >=1970 and the value is non-negative, the value is related to a
+ * Coordinated Universal Time name according to the C-language expression, where
+ * tm_sec, tm_min, tm_hour, tm_yday, and tm_year are all integer types:
+ * tm_sec + tm_min*60 + tm_hour*3600 + tm_yday*86400 +
+ *     (tm_year-70)*31536000 + ((tm_year-69)/4)*86400 -
+ *     ((tm_year-1)/100)*86400 + ((tm_year+299)/400)*86400
+ * OR
+ * ((((tm_year-69)/4 - (tm_year-1)/100 + (tm_year+299)/400 +
+ *         (tm_year-70)*365 + tm_yday)*24 + tm_hour)*60 + tm_min)*60 + tm_sec
+ */
+/* modified from %z case by hoisting offset outside if block and initializing */
+	  {
+	    long offset = 0;	/* offset < 0 => W of GMT, > 0 => E of GMT:
+				   subtract to get UTC */
+
+	    if (tim_p->tm_isdst >= 0)
+	      {
+		TZ_LOCK;
+		if (!tzset_called)
+		  {
+		    _tzset_unlocked ();
+		    tzset_called = 1;
+		  }
+
+#if defined (__CYGWIN__)
+		/* Cygwin must check if the application has been built with or
+		   without the extra tm members for backward compatibility, and
+		   then use either that or the old method fetching from tzinfo.
+		   Rather than pulling in the version check infrastructure, we
+		   just call a Cygwin function. */
+		extern long __cygwin_gettzoffset (const struct tm *tmp);
+		offset = __cygwin_gettzoffset (tim_p);
+#elif defined (__TM_GMTOFF)
+		offset = tim_p->__TM_GMTOFF;
+#else
+		__tzinfo_type *tz = __gettzinfo ();
+		/* The sign of this is exactly opposite the envvar TZ.  We
+		   could directly use the global _timezone for tm_isdst==0,
+		   but have to use __tzrule for daylight savings.  */
+		offset = -tz->__tzrule[tim_p->tm_isdst > 0].offset;
+#endif
+		TZ_UNLOCK;
+	      }
+	    len = snprintf (&s[count], maxsize - count, CQ("%lld"),
+			    (((((long long)tim_p->tm_year - 69)/4
+				- (tim_p->tm_year - 1)/100
+				+ (tim_p->tm_year + 299)/400
+				+ (tim_p->tm_year - 70)*365 + tim_p->tm_yday)*24
+			      + tim_p->tm_hour)*60 + tim_p->tm_min)*60
+			    + tim_p->tm_sec - offset);
+	    CHECK_LENGTH ();
+	  }
+          break;
 	case CQ('S'):
 #ifdef _WANT_C99_TIME_FORMATS
 	  if (alt != CQ('O') || !*alt_digits
@@ -1283,12 +1338,31 @@ recurse:
           if (tim_p->tm_isdst >= 0)
             {
 	      long offset;
-	      __tzinfo_type *tz = __gettzinfo ();
+
 	      TZ_LOCK;
+	      if (!tzset_called)
+		{
+		  _tzset_unlocked ();
+		  tzset_called = 1;
+		}
+
+#if defined (__CYGWIN__)
+	      /* Cygwin must check if the application has been built with or
+		 without the extra tm members for backward compatibility, and
+		 then use either that or the old method fetching from tzinfo.
+		 Rather than pulling in the version check infrastructure, we
+		 just call a Cygwin function. */
+	      extern long __cygwin_gettzoffset (const struct tm *tmp);
+	      offset = __cygwin_gettzoffset (tim_p);
+#elif defined (__TM_GMTOFF)
+	      offset = tim_p->__TM_GMTOFF;
+#else
+	      __tzinfo_type *tz = __gettzinfo ();
 	      /* The sign of this is exactly opposite the envvar TZ.  We
-	         could directly use the global _timezone for tm_isdst==0,
-	         but have to use __tzrule for daylight savings.  */
+		 could directly use the global _timezone for tm_isdst==0,
+		 but have to use __tzrule for daylight savings.  */
 	      offset = -tz->__tzrule[tim_p->tm_isdst > 0].offset;
+#endif
 	      TZ_UNLOCK;
 	      len = snprintf (&s[count], maxsize - count, CQ("%+03ld%.2ld"),
 			      offset / SECSPERHOUR,
@@ -1299,13 +1373,33 @@ recurse:
 	case CQ('Z'):
 	  if (tim_p->tm_isdst >= 0)
 	    {
-	      int size;
+	      size_t size;
+	      const char *tznam = NULL;
+
 	      TZ_LOCK;
-	      size = strlen(_tzname[tim_p->tm_isdst > 0]);
+	      if (!tzset_called)
+		{
+		  _tzset_unlocked ();
+		  tzset_called = 1;
+		}
+#if defined (__CYGWIN__)
+	      /* See above. */
+	      extern const char *__cygwin_gettzname (const struct tm *tmp);
+	      tznam = __cygwin_gettzname (tim_p);
+#elif defined (__TM_ZONE)
+	      tznam = tim_p->__TM_ZONE;
+#endif
+	      if (!tznam)
+		tznam = _tzname[tim_p->tm_isdst > 0];
+	      /* Note that in case of wcsftime this loop only works for
+	         timezone abbreviations using the portable codeset (aka ASCII).
+		 This seems to be the case, but if that ever changes, this
+		 loop needs revisiting. */
+	      size = strlen (tznam);
 	      for (i = 0; i < size; i++)
 		{
 		  if (count < maxsize - 1)
-		    s[count++] = _tzname[tim_p->tm_isdst > 0][i];
+		    s[count++] = tznam[i];
 		  else
 		    {
 		      TZ_UNLOCK;
@@ -1335,16 +1429,57 @@ recurse:
   return count;
 }
 
+size_t
+strftime (CHAR *__restrict s,
+	size_t maxsize,
+	const CHAR *__restrict format,
+	const struct tm *__restrict tim_p)
+{
+#ifdef _WANT_C99_TIME_FORMATS
+  era_info_t *era_info = NULL;
+  alt_digits_t *alt_digits = NULL;
+  size_t ret = __strftime (s, maxsize, format, tim_p, __get_current_locale (),
+			   &era_info, &alt_digits);
+  if (era_info)
+    free_era_info (era_info);
+  if (alt_digits)
+    free_alt_digits (alt_digits);
+  return ret;
+#else /* !_WANT_C99_TIME_FORMATS */
+  return __strftime (s, maxsize, format, tim_p, __get_current_locale (),
+		     NULL, NULL);
+#endif /* !_WANT_C99_TIME_FORMATS */
+}
+
+size_t
+strftime_l (CHAR *__restrict s, size_t maxsize, const CHAR *__restrict format,
+	    const struct tm *__restrict tim_p, struct __locale_t *locale)
+{
+#ifdef _WANT_C99_TIME_FORMATS
+  era_info_t *era_info = NULL;
+  alt_digits_t *alt_digits = NULL;
+  size_t ret = __strftime (s, maxsize, format, tim_p, locale,
+			   &era_info, &alt_digits);
+  if (era_info)
+    free_era_info (era_info);
+  if (alt_digits)
+    free_alt_digits (alt_digits);
+  return ret;
+#else /* !_WANT_C99_TIME_FORMATS */
+  return __strftime (s, maxsize, format, tim_p, locale, NULL, NULL);
+#endif /* !_WANT_C99_TIME_FORMATS */
+}
+
 /* The remainder of this file can serve as a regression test.  Compile
  *  with -D_REGRESSION_TEST.  */
 #if defined(_REGRESSION_TEST)	/* [Test code:  */
- 
+
 /* This test code relies on ANSI C features, in particular on the ability
  * of adjacent strings to be pasted together into one string.  */
- 
+
 /* Test output buffer size (should be larger than all expected results) */
 #define OUTSIZE	256
- 
+
 struct test {
 	CHAR  *fmt;	/* Testing format */
 	size_t  max;	/* Testing maxsize */
@@ -1356,9 +1491,9 @@ struct list {
 	const struct test *vec;	/* Test vectors */
 	int  cnt;		/* Number of vectors */
 	};
- 
+
 const char  TZ[]="TZ=EST5EDT";
- 
+
 /* Define list of test inputs and expected outputs, for the given time zone
  * and time.  */
 const struct tm  tm0 = {
@@ -1402,6 +1537,7 @@ const struct test  Vec0[] = {
 	{ CQ("%p"), 2+1, EXP(CQ("AM")) },
 	{ CQ("%r"), 11+1, EXP(CQ("09:53:47 AM")) },
 	{ CQ("%R"), 5+1, EXP(CQ("09:53")) },
+	{ CQ("%s"), 2+1, EXP(CQ("1230648827")) },
 	{ CQ("%S"), 2+1, EXP(CQ("47")) },
 	{ CQ("%t"), 1+1, EXP(CQ("\t")) },
 	{ CQ("%T"), 8+1, EXP(CQ("09:53:47")) },
@@ -1462,6 +1598,7 @@ const struct test  Vec1[] = {
 	{ CQ("%p"), 2+1, EXP(CQ("PM")) },
 	{ CQ("%r"), 11+1, EXP(CQ("11:01:13 PM")) },
 	{ CQ("%R"), 5+1, EXP(CQ("23:01")) },
+	{ CQ("%s"), 2+1, EXP(CQ("1215054073")) },
 	{ CQ("%S"), 2+1, EXP(CQ("13")) },
 	{ CQ("%t"), 1+1, EXP(CQ("\t")) },
 	{ CQ("%T"), 8+1, EXP(CQ("23:01:13")) },
@@ -1486,7 +1623,7 @@ const struct test  Vec1[] = {
 	#undef VEC
 	#undef EXP
 	};
- 
+
 #if YEAR_BASE == 1900  /* ( */
 /* Checks for very large years.  YEAR_BASE value relied upon so that the
  * answer strings can be predetermined.
@@ -1584,7 +1721,7 @@ const struct test  Vecyr1[] = {
 #undef CENT
 #undef Year
 #endif /* YEAR_BASE ) */
- 
+
 /* Checks for years just over zero (also test for s=60).
  * Years less than 4 digits are not mentioned for %Y in the standard, so the
  * test for that case is based on the design intent.  */
@@ -1655,7 +1792,7 @@ const struct test  Vecyrzn[] = {
 #undef YEAR
 #undef CENT
 #undef Year
- 
+
 const struct list  ListYr[] = {
 	{ &tmyrzp, Vecyrzp, sizeof(Vecyrzp)/sizeof(Vecyrzp[0]) },
 	{ &tmyrzn, Vecyrzn, sizeof(Vecyrzn)/sizeof(Vecyrzn[0]) },
@@ -1664,19 +1801,19 @@ const struct list  ListYr[] = {
 	{ &tmyr1, Vecyr1, sizeof(Vecyr1)/sizeof(Vecyr1[0]) },
 	#endif
 	};
- 
- 
+
+
 /* List of tests to be run */
 const struct list  List[] = {
 	{ &tm0, Vec0, sizeof(Vec0)/sizeof(Vec0[0]) },
 	{ &tm1, Vec1, sizeof(Vec1)/sizeof(Vec1[0]) },
 	};
- 
+
 #if defined(STUB_getenv_r)
 char *
 _getenv_r(struct _reent *p, const char *cp) { return getenv(cp); }
 #endif
- 
+
 int
 main(void)
 {
@@ -1684,7 +1821,7 @@ int  i, l, errr=0, erro=0, tot=0;
 const char  *cp;
 CHAR  out[OUTSIZE];
 size_t  ret;
- 
+
 /* Set timezone so that %z and %Z tests come out right */
 cp = TZ;
 if((i=putenv(cp)))  {
@@ -1696,7 +1833,7 @@ if(strcmp(getenv("TZ"),strchr(TZ,'=')+1))  {
     return(-2);
     }
 tzset();
- 
+
 #if defined(VERBOSE)
 printf("_timezone=%d, _daylight=%d, _tzname[0]=%s, _tzname[1]=%s\n", _timezone, _daylight, _tzname[0], _tzname[1]);
 {
@@ -1708,7 +1845,7 @@ __tzinfo_type *tz = __gettzinfo ();
 printf("tz->__tzrule[0].offset=%d, tz->__tzrule[1].offset=%d\n", tz->__tzrule[0].offset, tz->__tzrule[1].offset);
 }
 #endif
- 
+
 /* Run all of the exact-length tests as-given--results should match */
 for(l=0; l<sizeof(List)/sizeof(List[0]); l++)  {
     const struct list  *test = &List[l];
@@ -1729,7 +1866,7 @@ for(l=0; l<sizeof(List)/sizeof(List[0]); l++)  {
 	    }
 	}
     }
- 
+
 /* Run all of the exact-length tests with the length made too short--expect to
  * fail.  */
 for(l=0; l<sizeof(List)/sizeof(List[0]); l++)  {
@@ -1755,7 +1892,7 @@ for(l=0; l<sizeof(List)/sizeof(List[0]); l++)  {
 	    }
 	}
     }
- 
+
 /* Run all of the special year test cases */
 for(l=0; l<sizeof(ListYr)/sizeof(ListYr[0]); l++)  {
     const struct list  *test = &ListYr[l];
@@ -1776,14 +1913,14 @@ for(l=0; l<sizeof(ListYr)/sizeof(ListYr[0]); l++)  {
 	    }
 	}
     }
- 
+
 #define STRIZE(f)	#f
 #define NAME(f)	STRIZE(f)
 printf(NAME(strftime) "() test ");
 if(errr || erro)  printf("FAILED %d/%d of", errr, erro);
   else    printf("passed");
 printf(" %d test cases.\n", tot);
- 
+
 return(errr || erro);
 }
 #endif /* defined(_REGRESSION_TEST) ] */
